// smart_assert_loggers.hpp

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_LOGGERS_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_LOGGERS_HPP_INCLUDED

// Default classes that do the logging,
// in case an ASSERT fails

#include <sstream>
#include <fstream>
#include <algorithm>
#include <utility>
#include "smart_assert_context.hpp"
#include "smart_assert_defs.hpp"
#include "smart_assert_util.hpp"

namespace boost { 

namespace smart_assert {


// FIXME - this SHOULD NOT be a policy !!!
// should be a function/ functor, which should be kept in
// smart_assert_settings class.
class default_logger {

public:
    void log( const assert_context & context) {
        std::ostringstream out;
        dump_all_assert_context( context, out);
        log_from_stream( out);
    }

private:
    static std::ostream & get_log() {
        // FIXME - initialize it before main!
        static std::ofstream out( "./asserts.txt");
        return out;
    }

    void log_from_stream( const std::ostringstream & out) {
        // note: we don't need to be too fast...
        // we do logging only when an assertion fails,
        // which should not be too often ;-)

        std::ostream & log = get_log();
        log << out.str();
        // FIXME - make thread-safe, when needed!

        // note: you should always flush (for instance, the assertion
        // might be fatal, so the program might crash anytime now)
        log.flush();
    }
};


} // namespace smart_assert

} // namespace boost

#endif